close all
clear
clc

physical_constants; %constant for c0

res1_val = 2e3; %ohm  
res2_val = 4e3; %ohm 

unit = 0.001;  %mm
mm = 1;  %cm

f_max = 10e6;
f_excite = 30e6;

wire_rad = 8*mm; %quite a bit larger than Romer
mesh_size = 8*mm;   %adjust this for trade-off between speed and resolution of the mesh
mesh_subsamp = 1;  %resolution needs to be 1 or less for decent material mesh passed on to FDTD engine

solenoid_length = 500*mm;  %fix this. replace with coil length or vice versa 
port_length = 50*mm; 
port_resist = 5; %5 ohm feed resistor
resistor_offset = 8*mm; 


coil_res = 60; %coil resolution for straight-wire segments around loop 
coil_rad = 100*mm; 
coil_length = 1000*mm;
coil_turns = 48;
wire_rad2 = wire_rad;
coil_res2 = coil_res;  
coil_rad2 = coil_rad * 4;

%% define openEMS options 
openEMS_opts = ' --debug-PEC';

Sim_Path = 'emfloop_tmp';
Sim_CSX = 'emfloop_sim.xml';

[status, message, messageid] = rmdir(Sim_Path,'s');
[status,message,messageid] = mkdir(Sim_Path);

%% setup FDTD parameter & excitation function 
%FDTD = InitFDTD('NrTS',30000*10,'EndCriteria',1e-6);  %stop when the energy level falls by 60dB or 6000 time steps
FDTD = InitFDTD('NrTS',28800);  %stop when the energy level falls by 60dB or 6000 time steps
%FDTD = SetGaussExcite(FDTD,f_excite/2,f_excite/2);  %gaussian pulse applied to the excitation coil to get the simulation started
FDTD = SetStepExcite(FDTD, f_max);  

BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8'}; % use pml to smoothly absorb fields on 8 outer layers of sim boundary 
FDTD = SetBoundaryCond( FDTD, BC );

CSX = InitCSX();

%CSX = AddMetal(CSX,'copper');  %pec dump testing
CSX = AddMaterial(CSX,'copper');
CSX = SetMaterialProperty(CSX,'copper','Kappa',56e6); %conductivity of typical copper material (S/m)

%CSX = AddMaterial(CSX,'super_conductor');
%CSX = SetMaterialProperty(CSX,'super_conductor','Kappa', 1e9, 'Mue', 1e-3); 

%CSX = AddMaterial(CSX,'nichrome');
%CSX = SetMaterialProperty(CSX,'nichrome','Kappa',6.7e5); %conductivity of typical nichrome material (S/m)

%CSX = AddMaterial(CSX,'graphite_carbon');
%CSX = SetMaterialProperty(CSX,'graphite_carbon','Kappa',3.3e2); %conductivity of carbon  (S/m)

%CSX = AddMaterial(CSX,'silver');
%CSX = SetMaterialProperty(CSX,'silver','Kappa',6.3e7); %conductivity of silver  (S/m)

%build the excitation coil
dt = 1.0/coil_res;
height=0;
wire.Vertex = {};


count = 0;
p6(1,count+1) = coil_rad*1.2;
p6(2,count+1) = 0;
p6(3,count+1) = -port_length/2;

p6(1,count+2) = coil_rad*1.2;
p6(2,count+2) = 0;
p6(3,count+2) = -coil_length/2;

p6(1,count+3) = coil_rad-wire_rad/2; 
p6(2,count+3) = 0;
p6(3,count+3) = -coil_length/2;

count = 0;
for n=0:coil_turns-1
     for m=0:coil_res;
        count = count + 1;
        p7(1,count) = coil_rad * cos(2*pi*dt*m);
        p7(2,count) = coil_rad * sin(2*pi*dt*m);
        p7(3,count) = height + coil_length/coil_turns * dt*m - coil_length/2;
    end
    height = height + coil_length/coil_turns;
end

count=0;
p8(1,count+1) = coil_rad-wire_rad/2; 
p8(2,count+1) = 0;
p8(3,count+1) = coil_length/2;

p8(1,count+2) = coil_rad*1.2;
p8(2,count+2) = 0;
p8(3,count+2) = coil_length/2;
p8(1,count+3) = coil_rad*1.2;
p8(2,count+3) = 0;
p8(3,count+3) = port_length/2; 

CSX = AddWire(CSX, 'copper', 0, p7, wire_rad);
CSX = AddWire(CSX, 'copper', 0, p6, wire_rad);
CSX = AddWire(CSX, 'copper', 0, p8, wire_rad);

%pass 2 to fill in segment gaps
count = 0;
height = 0;
for n=0:coil_turns-1
     for m=1/8:coil_res;
        count = count + 1;
        p9(1,count) = coil_rad * cos(2*pi*dt*m);
        p9(2,count) = coil_rad * sin(2*pi*dt*m);
        p9(3,count) = height + coil_length/coil_turns * dt*m - coil_length/2;
    end
    height = height + coil_length/coil_turns;
end
CSX = AddWire(CSX, 'copper', 0, p9, wire_rad);

antenna_offset=0;
antenna_y_off=0;

%port 7 excitation,  this is the coil feed, compenstate less turns with higher current
%excitation port = true
start=[coil_rad*1.2+-wire_rad/2 -wire_rad/2 -port_length];
stop=[coil_rad*1.2+ wire_rad/2 wire_rad/2 +port_length];
CSX = AddLumpedPort(CSX,5,7,port_resist, start, stop, [0 0 50], true);  %50 V for roamer sim 


%build the experiment coil, resistors 
%pass 1
dt = 1.0/coil_res2;
wire.Vertex = {};
count=0;
    for m=0:coil_res2/2
        count = count + 1;
        p2(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p2(2,count) = coil_rad2 * sin(2*pi*dt*m) + resistor_offset/4;
        p2(3,count) = 0; 
    end
        count = 1;
        p4(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p4(2,count) = coil_rad2 * sin(2*pi*dt*m) + resistor_offset/4;
        p4(3,count) = 0; 
        m = m+1;
        count = 2; 
        p4(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p4(2,count) = coil_rad2 * sin(2*pi*dt*m) + resistor_offset/4;
        p4(3,count) = 0; 

    CSX = AddWire(CSX, 'copper', 0, p2, wire_rad2);

count=0;
    
    for m=coil_res2/2:coil_res2
        count = count + 1;
        p3(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p3(2,count) = coil_rad2 * sin(2*pi*dt*m) - resistor_offset/4;
        p3(3,count) = 0; 
    end
        count = 1;
        p5(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p5(2,count) = coil_rad2 * sin(2*pi*dt*m) - resistor_offset/4;
        p5(3,count) = 0; 
        m = m+1;
        count = 2; 
        p5(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p5(2,count) = coil_rad2 * sin(2*pi*dt*m) - resistor_offset/4;
        p5(3,count) = 0; 

    CSX = AddWire(CSX, 'copper', 0, p3, wire_rad2);

%build the experiment coil, resistors 
%pass 2, pass 2 fills in the gaps created by straight wire segments
dt = 1.0/coil_res2;
wire.Vertex = {};
count=0;
    for m=1/8:coil_res2/2
        count = count + 1;
        p2(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p2(2,count) = coil_rad2 * sin(2*pi*dt*m) + resistor_offset/4;
        p2(3,count) = 0; 
    end
        count = 1;
        p4(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p4(2,count) = coil_rad2 * sin(2*pi*dt*m) + resistor_offset/4;
        p4(3,count) = 0; 
        m = m+1;
        count = 2; 
        p4(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p4(2,count) = coil_rad2 * sin(2*pi*dt*m) + resistor_offset/4;
        p4(3,count) = 0; 

    CSX = AddWire(CSX, 'copper', 0, p2, wire_rad2);

count=0;
    
    for m=coil_res2/2+1/8:coil_res2
        count = count + 1;
        p3(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p3(2,count) = coil_rad2 * sin(2*pi*dt*m) - resistor_offset/4;
        p3(3,count) = 0; 
    end
        count = 1;
        p5(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p5(2,count) = coil_rad2 * sin(2*pi*dt*m) - resistor_offset/4;
        p5(3,count) = 0; 
        m = m+1;
        count = 2; 
        p5(1,count) = coil_rad2 * cos(2*pi*dt*m);
        p5(2,count) = coil_rad2 * sin(2*pi*dt*m) - resistor_offset/4;
        p5(3,count) = 0; 

    CSX = AddWire(CSX, 'copper', 0, p3, wire_rad2);


%experiment res1,
x11 = p4(1,1);
y11 = +resistor_offset*2;
z11 = p4(3,1);
x21 = p4(1,2);
y21 = -resistor_offset*2;
z21 = p4(3,2);
CSX = AddLumpedPort(CSX,5,1,res1_val,[x21-wire_rad2/2.0,y11,-wire_rad2/2.0], [x11+wire_rad2/2.0,y21,wire_rad2/2.0],[0 1 0], false); 

%experiment res2, 
x12 = p5(1,1);
y12 = +resistor_offset*2;
z12 = p5(3,1);
x22 = p5(1,2);
y22 = -resistor_offset*2;
z22 = p5(3,2);
CSX = AddLumpedPort(CSX,5,2,res2_val,[x12-wire_rad2/2.0,y12,-wire_rad2/2.0], [x22+wire_rad2/2.0,y22,wire_rad2/2.0],[0 1 0], false); 

probe_pullback = 8;
probe1(1,1) = mesh_size*probe_pullback;
probe1(2,1) = coil_rad2-wire_rad/2;
probe1(3,1) = 0; 
probe1(1,2) = coil_rad2*1.5; 
probe1(2,2) = coil_rad2-wire_rad/2;
probe1(3,2) = 0; 
CSX = AddWire(CSX, 'copper', 0, probe1, wire_rad2);

probe2(1,1) = -mesh_size*probe_pullback;
probe2(2,1) = coil_rad2-wire_rad/2;
probe2(3,1) = 0; 
probe2(1,2) = -coil_rad2*1.5; 
probe2(2,2) = coil_rad2-wire_rad/2;
probe2(3,2) = 0; 
CSX = AddWire(CSX, 'copper', 0, probe2, wire_rad2);

probe3(1,1) = mesh_size*probe_pullback;
probe3(2,1) = -coil_rad2+wire_rad/2;
probe3(3,1) = 0; 
probe3(1,2) = coil_rad2*1.5; 
probe3(2,2) = -coil_rad2+wire_rad/2;
probe3(3,2) = 0; 
CSX = AddWire(CSX, 'copper', 0, probe3, wire_rad2);

probe4(1,1) = -mesh_size*probe_pullback;
probe4(2,1) = -coil_rad2+wire_rad/2;
probe4(3,1) = 0; 
probe4(1,2) = -coil_rad2*1.5; 
probe4(2,2) = -coil_rad2+wire_rad/2;
probe4(3,2) = 0; 
CSX = AddWire(CSX, 'copper', 0, probe4, wire_rad2);

CSX = AddLumpedPort(CSX,5,5,20e6,[coil_rad2*1.5-wire_rad2*2 -coil_rad2+wire_rad/2 -wire_rad2/2], [coil_rad2*1.5 coil_rad2-wire_rad/2 +wire_rad2/2],[0 1 0], false); 
CSX = AddLumpedPort(CSX,5,6,20e6,[-coil_rad2*1.5 -coil_rad2+wire_rad/2 -wire_rad2/2], [-coil_rad2*1.5+wire_rad2*2 coil_rad2-wire_rad/2 +wire_rad2/2],[0 1 0], false); 

%mesh with log spacing
add_Lines = 8 * mesh_size * 1.5.^(1:10);  %log response, decreasing res toward boundary of mesh
add_Lines = add_Lines(find(add_Lines<10*1e3));

mesh.x = -coil_rad2*2-mesh_size : mesh_size*mesh_subsamp : coil_rad2*2+mesh_size;    %higher resolution around the coils
mesh.x = [mesh.x(1)-add_Lines mesh.x mesh.x(end)+add_Lines ];

mesh.y = -coil_rad2*1.1-mesh_size : mesh_size*mesh_subsamp : coil_rad2*1.1+mesh_size;
mesh.y = [mesh.y(1)-add_Lines mesh.y mesh.y(end)+add_Lines ];

mesh.z = -mesh_size*90 : mesh_size*mesh_subsamp/3 : mesh_size*90;  %more dense in the z dir
mesh.z = [mesh.z(1)-add_Lines mesh.z mesh.z(end)+add_Lines ];

CSX = DefineRectGrid(CSX, unit, mesh);

%total volume
%dump box area
start = [-solenoid_length, -solenoid_length, -solenoid_length];
stop = [solenoid_length, solenoid_length, solenoid_length];

%total volume
%The Et dump files can be viewed as an animation over time of the changing electric field using Paraview
%CSX = AddDump(CSX,'Et_','DumpType',0,'DumpMode',2);
%CSX = AddBox(CSX,'Et_',0 , start,stop);

%total volume
%The Ht dump files can be viewed as an animation over time of the changing magnetic field using Paraview
%CSX = AddDump(CSX,'Ht_','DumpType',1,'DumpMode',2);
%CSX = AddBox(CSX,'Ht_',0 , start,stop);

start_slice = [-solenoid_length*1.5, -solenoid_length*1.5, 0]; 
stop_slice = [solenoid_length*1.5, solenoid_length*1.5, 0]; 
%The Ht dump / 2d slice horizontal 
CSX = AddDump(CSX,'Ht_horiz_','DumpType',1,'DumpMode',2);
CSX = AddBox(CSX,'Ht_horiz_',0 , start_slice, stop_slice);

start_slice = [-solenoid_length*1.5, 0 -solenoid_length*1.5]; 
stop_slice = [solenoid_length*1.5, 0 solenoid_length*1.5]; 
%The Ht dump / 2d slice vertical 
CSX = AddDump(CSX,'Ht_vert_','DumpType',1,'DumpMode',2);
CSX = AddBox(CSX,'Ht_vert_',0 , start_slice, stop_slice);


start_slice = [-solenoid_length*1.5, -solenoid_length*1.5, 0]; 
stop_slice = [solenoid_length*1.5, solenoid_length*1.5, 0]; 
%The Et dump / 2d slice horizontal 
CSX = AddDump(CSX,'Et_horiz_','DumpType',0,'DumpMode',2);
CSX = AddBox(CSX,'Et_horiz_',0 , start_slice, stop_slice);

start_slice = [-solenoid_length*1.5, 0 -solenoid_length*1.5]; 
stop_slice = [solenoid_length*1.5, 0 solenoid_length*1.5]; 
%The Et dump / 2d slice vertical 
CSX = AddDump(CSX,'Et_vert_','DumpType',0,'DumpMode',2);
CSX = AddBox(CSX,'Et_vert_',0 , start_slice, stop_slice);


%% Write openEMS compatable xml-file %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
WriteOpenEMS([Sim_Path '/' Sim_CSX],FDTD,CSX);

%% run openEMS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
RunOpenEMS(Sim_Path, Sim_CSX, openEMS_opts);
